__title__ = 'Cyclosible'
__version__ = '0.3'
__author__ = 'Cycloid'
__license__ = 'Apache'
__copyright__ = 'Copyright 2015'

# Version synonym
VERSION = __version__

# Header encoding (see RFC5987)
HTTP_HEADER_ENCODING = 'iso-8859-1'

# Default datetime input and output formats
ISO_8601 = 'iso-8601'
